@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map Functions
- * Copyright (C) 2005-2009 Red Hat Inc.
+ * Copyright (C) 2005-2009, 2012 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -15,14 +15,26 @@
  * @brief Implements maps (associative arrays) and lists
  */
 
-#include "sym.c"
 #include "stat-common.c"
 #include "map-stat.c"
 
+#if NEED_MAP_LOCKS
+#ifdef __KERNEL__
+#define MAP_LOCK(m)	spin_lock(&(m)->lock)
+#define MAP_UNLOCK(m)	spin_unlock(&(m)->lock)
+#else
+#define MAP_LOCK(m)	pthread_mutex_lock(&(m)->lock)
+#define MAP_UNLOCK(m)	pthread_mutex_unlock(&(m)->lock)
+#endif
+#else
+#define MAP_LOCK(m)	do {} while (0)
+#define MAP_UNLOCK(m)	do {} while (0)
+#endif
+
 static int map_sizes[] = {
         sizeof(int64_t),
         MAP_STRING_LENGTH,
-        sizeof(stat),
+        sizeof(stat_data),
         0
 };
 
@@ -110,11 +122,11 @@ static char *_stp_get_str(struct map_node *m)
  * @param m pointer to the map_node.
  * @returns A pointer to the stats.  
  */
-static stat *_stp_get_stat(struct map_node *m)
+static stat_data *_stp_get_stat(struct map_node *m)
 {
 	if (!m || m->map->type != STAT)
 		return 0;
-	return (stat *)((long)m + m->map->data_offset);
+	return (stat_data *)((long)m + m->map->data_offset);
 }
 
 /** Return an int64 key from a map node.
@@ -170,7 +182,9 @@ static char *_stp_key_get_str (struct map_node *mn, int n)
  * @ingroup map_create
  */
 
-static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, int data_size, int cpu)
+static int
+_stp_map_init(MAP m, unsigned max_entries, int wrap, int type, int key_size,
+	      int data_size, int cpu)
 {
 	int size;
 	size_t hash_size = sizeof(struct hlist_head) * HASH_TABLE_SIZE;
@@ -185,6 +199,7 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 	memset(m->hashes, 0, hash_size);
 	m->maxnum = max_entries;
 	m->type = type;
+	m->wrap = wrap;
 	if (type >= END) {
 		_stp_error("unknown map type %d\n", type);
 		return -1;
@@ -195,11 +210,11 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 		
 		/* size is the size of the map_node. */
 		/* add space for the value. */
-		key_size = ALIGN(key_size,4);
+		key_size = (key_size + 3) & ~3; //ALIGN(key_size,4);
 		m->data_offset = key_size;
 		if (data_size == 0)
 			data_size = map_sizes[type];
-		data_size = ALIGN(data_size,4);
+		data_size = (data_size + 3) & ~3; //ALIGN(data_size,4);
 		size = key_size + data_size;
 		
 		
@@ -210,8 +225,10 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 			else
 				tmp = _stp_kmalloc_node_gfp(size, cpu_to_node(cpu), STP_ALLOC_SLEEP_FLAGS);
 		
-			if (!tmp)
+			if (!tmp) {
+				_stp_error("error allocating map entry\n");
 				return -1;
+			}
 			
 //			dbug ("allocated %lx\n", (long)tmp);
 			list_add((struct list_head *)tmp, &m->pool);
@@ -224,23 +241,73 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 }
 
 
-static MAP _stp_map_new(unsigned max_entries, int type, int key_size, int data_size)
+static MAP
+_stp_map_new(unsigned max_entries, int wrap, int type, int key_size,
+	     int data_size)
 {
 	/* Called from module_init, so user context, may sleep alloc. */
-	MAP m = (MAP) _stp_kzalloc_gfp(sizeof(struct map_root), STP_ALLOC_SLEEP_FLAGS);
+	MAP m = (MAP) _stp_kzalloc_gfp(sizeof(struct map_root),
+				       STP_ALLOC_SLEEP_FLAGS);
 	if (m == NULL)
 		return NULL;
 
 	INIT_LIST_HEAD(&m->pool);
 	INIT_LIST_HEAD(&m->head);
-	if (_stp_map_init(m, max_entries, type, key_size, data_size, -1)) {
+	if (_stp_map_init(m, max_entries, wrap, type, key_size, data_size,
+			  -1)) {
 		_stp_map_del(m);
 		return NULL;
 	}
 	return m;
 }
 
-static PMAP _stp_pmap_new(unsigned max_entries, int type, int key_size, int data_size)
+#ifndef __KERNEL__
+static int _stp_map_tls_object_init(struct tls_data_object_t *obj)
+{
+	MAP m = container_of(obj, struct map_root, object);
+	PMAP p = container_of(obj->container, struct pmap, container);
+
+	INIT_LIST_HEAD(&m->pool);
+	INIT_LIST_HEAD(&m->head);
+	m->hashes = NULL;
+
+#if NEED_MAP_LOCKS
+	{
+		int rc;
+		if ((rc = pthread_mutex_init(&m->lock, NULL)) != 0) {
+			_stp_error("Couldn't initialize map mutex: %d\n", rc);
+			return -1;
+		}
+	}
+#endif
+
+	/* To get the correct parameters for _stp_map_init(), get them
+	 * from the cached values in PMAP. */
+	if (_stp_map_init(m, p->max_entries, p->wrap, p->type, p->key_size,
+			  p->data_size, -1) != 0) {
+		__stp_map_del(m);
+#if NEED_MAP_LOCKS
+		(void)pthread_mutex_destroy(&m->lock);
+#endif
+		return -1;
+	}
+
+	return 0;
+}
+
+static void _stp_map_tls_object_free(struct tls_data_object_t *obj)
+{
+	MAP m = container_of(obj, struct map_root, object);
+	__stp_map_del(m);
+#if NEED_MAP_LOCKS
+	(void)pthread_mutex_destroy(&m->lock);
+#endif
+}
+#endif
+
+static PMAP
+_stp_pmap_new(unsigned max_entries, int wrap, int type, int key_size,
+	      int data_size)
 {
 	int i;
 	MAP map, m;
@@ -250,38 +317,63 @@ static PMAP _stp_pmap_new(unsigned max_entries, int type, int key_size, int data
 	if (pmap == NULL)
 		return NULL;
 
+#ifdef __KERNEL__
 	pmap->map = map = (MAP) _stp_alloc_percpu (sizeof(struct map_root));
 	if (map == NULL) 
 		goto err;
+#else
+	if (_stp_tls_data_container_init(&pmap->container,
+					 sizeof(struct map_root),
+					 &_stp_map_tls_object_init,
+					 &_stp_map_tls_object_free) != 0)
+		goto err;
+#endif
 
+#ifdef __KERNEL__
 	/* initialize the memory lists first so if allocations fail */
         /* at some point, it is easy to clean up. */
-	stp_for_each_cpu(i) {
+	for_each_possible_cpu(i) {
 		m = per_cpu_ptr (map, i);
 		INIT_LIST_HEAD(&m->pool);
 		INIT_LIST_HEAD(&m->head);
 	}
+#endif
 	INIT_LIST_HEAD(&pmap->agg.pool);
 	INIT_LIST_HEAD(&pmap->agg.head);
 
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		m = per_cpu_ptr (map, i);
-		if (_stp_map_init(m, max_entries, type, key_size, data_size, i)) {
+		if (_stp_map_init(m, max_entries, wrap, type, key_size,
+				  data_size, i)) {
 			goto err1;
 		}
 	}
+#else
+	/* Cache values for use by _stp_map_tls_object_init(). */
+	pmap->max_entries = max_entries;
+	pmap->type = type;
+	pmap->key_size = key_size;
+	pmap->data_size = data_size;
+	pmap->wrap = wrap;
+#endif
 
-	if (_stp_map_init(&pmap->agg, max_entries, type, key_size, data_size, -1))
+	if (_stp_map_init(&pmap->agg, max_entries, wrap, type, key_size,
+			  data_size, -1))
 		goto err1;
 	
 	return pmap;
 
 err1:
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		m = per_cpu_ptr (map, i);
 		__stp_map_del(m);
 	}
 	_stp_free_percpu(map);
+#else
+	_stp_tls_data_container_cleanup(&pmap->container);
+#endif
 err:
 	_stp_kfree(pmap);
 	return NULL;
@@ -364,16 +456,28 @@ static void _stp_pmap_clear(PMAP pmap)
 	if (pmap == NULL)
 		return;
 
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		MAP m = per_cpu_ptr (pmap->map, i);
-#if NEED_MAP_LOCKS
-		spin_lock(&m->lock);
-#endif
+
+		MAP_LOCK(m);
 		_stp_map_clear(m);
-#if NEED_MAP_LOCKS
-		spin_unlock(&m->lock);
-#endif
+		MAP_UNLOCK(m);
 	}
+#else
+	{
+		struct tls_data_object_t *obj;
+		TLS_DATA_CONTAINER_LOCK(&pmap->container);
+		for_each_tls_data(obj, &pmap->container) {
+			MAP m = container_of(obj, struct map_root, object);
+
+			MAP_LOCK(m);
+			_stp_map_clear(m);
+			MAP_UNLOCK(m);
+		}
+		TLS_DATA_CONTAINER_UNLOCK(&pmap->container);
+	}
+#endif
 	_stp_map_clear(&pmap->agg);
 }
 
@@ -418,11 +522,15 @@ static void _stp_pmap_del(PMAP pmap)
 	if (pmap == NULL)
 		return;
 
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		MAP m = per_cpu_ptr (pmap->map, i);
 		__stp_map_del(m);
 	}
 	_stp_free_percpu(pmap->map);
+#else
+	_stp_tls_data_container_cleanup(&pmap->container);
+#endif
 
 	/* free agg map elements */
 	__stp_map_del(&pmap->agg);
@@ -460,8 +568,8 @@ static int _stp_cmp (struct list_head *a, struct list_head *b, int keynum, int d
 			a = _stp_key_get_int64(m1, keynum);
 			b = _stp_key_get_int64(m2, keynum);
 		} else if (keynum < 0) {
-			stat *sd1 = (stat *)((long)m1 + m1->map->data_offset);
-			stat *sd2 = (stat *)((long)m2 + m2->map->data_offset);
+			stat_data *sd1 = (stat_data *)((long)m1 + m1->map->data_offset);
+			stat_data *sd2 = (stat_data *)((long)m2 + m2->map->data_offset);
 			switch (keynum) {
 			case SORT_COUNT:
 				a = sd1->count;
@@ -661,8 +769,8 @@ static struct map_node *_stp_new_agg(MAP agg, struct hlist_head *ahead, struct m
 				 0);
 		break;
 	case STAT: {
-		stat *sd1 = (stat *)((long)aptr + agg->data_offset);
-		stat *sd2 = (stat *)((long)ptr + ptr->map->data_offset);
+		stat_data *sd1 = (stat_data *)((long)aptr + agg->data_offset);
+		stat_data *sd2 = (stat_data *)((long)ptr + ptr->map->data_offset);
 		Hist st = &agg->hist;
 		sd1->count = sd2->count;
 		sd1->sum = sd2->sum;
@@ -697,8 +805,8 @@ static void _stp_add_agg(struct map_node *aptr, struct map_node *ptr)
 				 1);
 		break;
 	case STAT: {
-		stat *sd1 = (stat *)((long)aptr + aptr->map->data_offset);
-		stat *sd2 = (stat *)((long)ptr + ptr->map->data_offset);
+		stat_data *sd1 = (stat_data *)((long)aptr + aptr->map->data_offset);
+		stat_data *sd2 = (stat_data *)((long)ptr + ptr->map->data_offset);
 		Hist st = &aptr->map->hist;
 		if (sd1->count == 0) {
 			sd1->count = sd2->count;
@@ -741,6 +849,10 @@ static MAP _stp_pmap_agg (PMAP pmap)
 	struct map_node *ptr, *aptr = NULL;
 	struct hlist_head *head, *ahead;
 	struct hlist_node *e, *f;
+#ifndef __KERNEL__
+	struct tls_data_object_t *obj;
+#endif
+	int quit = 0;
 
 	agg = &pmap->agg;
 	
@@ -748,11 +860,15 @@ static MAP _stp_pmap_agg (PMAP pmap)
 	/* every time we aggregate. which would be best? */
 	_stp_map_clear (agg);
 
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		m = per_cpu_ptr (pmap->map, i);
-#if NEED_MAP_LOCKS
-		spin_lock(&m->lock);
+#else
+	TLS_DATA_CONTAINER_LOCK(&pmap->container);
+	for_each_tls_data(obj, &pmap->container) {
+		m = container_of(obj, struct map_root, object);
 #endif
+		MAP_LOCK(m);
 		/* walk the hash chains. */
 		for (hash = 0; hash < HASH_TABLE_SIZE; hash++) {
 			head = &m->hashes[hash];
@@ -771,18 +887,22 @@ static MAP _stp_pmap_agg (PMAP pmap)
 					_stp_add_agg(aptr, ptr);
 				else {
 					if (!_stp_new_agg(agg, ahead, ptr)) {
-#if NEED_MAP_LOCKS
-						spin_unlock(&m->lock);
-#endif
-						return NULL;
+                                                MAP_UNLOCK(m);
+                                                agg = NULL;
+						goto out;
+                                                // NB: break would head out to the for (hash...) 
+                                                // loop, which behaves badly with an agg==NULL.
 					}
 				}
 			}
 		}
-#if NEED_MAP_LOCKS
-		spin_unlock(&m->lock);
-#endif
+		MAP_UNLOCK(m);
 	}
+
+        out:
+#ifndef __KERNEL__
+	TLS_DATA_CONTAINER_UNLOCK(&pmap->container);
+#endif
 	return agg;
 }
 
@@ -809,7 +929,7 @@ static void _new_map_clear_node (struct map_node *m)
 		break;
 	case STAT: 
 	{
-		stat *sd = (stat *)((long)m + m->map->data_offset);
+		stat_data *sd = (stat_data *)((long)m + m->map->data_offset);
 		Hist st = &m->map->hist;
 		sd->count = 0;
 		if (st->type != HIST_NONE) {
@@ -885,12 +1005,12 @@ static int _new_map_set_str (MAP map, struct map_node *n, char *val, int add)
 
 static int _new_map_set_stat (MAP map, struct map_node *n, int64_t val, int add)
 {
-	stat *sd;
+	stat_data *sd;
 
 	if (map == NULL || n == NULL)
 		return -2;
 
-	sd = (stat *)((long)n + map->data_offset);
+	sd = (stat_data *)((long)n + map->data_offset);
 	if (!add) {
 		Hist st = &map->hist;
 		sd->count = 0;
@@ -924,10 +1044,20 @@ static int _stp_pmap_size (PMAP pmap)
 {
 	int i, num = 0;
 
-	stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
 		MAP m = per_cpu_ptr (pmap->map, i);
 		num += m->num;
 	}
+#else
+	struct tls_data_object_t *obj;
+	TLS_DATA_CONTAINER_LOCK(&pmap->container);
+	for_each_tls_data(obj, &pmap->container) {
+		MAP m = container_of(obj, struct map_root, object);
+		num += m->num;
+	}
+	TLS_DATA_CONTAINER_UNLOCK(&pmap->container);
+#endif
 	return num;
 }
 #endif /* _MAP_C_ */
