@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map Header File
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005, 2012 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -8,10 +8,16 @@
  * later version.
  */
 
-#include <linux/log2.h>
 #ifndef _MAP_H_
 #define _MAP_H_
 
+#if defined(__KERNEL__)
+#include <linux/log2.h>
+#elif defined(__DYNINST__)
+#include "dyninst/ilog2.h"
+#include "dyninst/tls_data.c"
+#endif
+
 /** @file map.h
  * @brief Header file for maps and lists 
  */
@@ -89,6 +95,11 @@ struct map_node {
  * It is allocated once when _stp_map_new() is called. 
  */
 struct map_root {
+#ifndef __KERNEL__
+	/* Note that the tls_data_object_t must be first in struct
+	 * map_root. */
+	struct tls_data_object_t object;
+#endif
 	/* type of the value stored in the array */
 	int type;
 	
@@ -123,7 +134,7 @@ struct map_root {
 
 	int data_offset;
 
-#ifdef NEED_MAP_LOCKS
+#if NEED_MAP_LOCKS
 	spinlock_t lock;
 #endif
 
@@ -138,7 +149,18 @@ struct map_root {
 typedef struct map_root *MAP;
 
 struct pmap {
+#ifdef __KERNEL__
 	MAP map;		/* per-cpu maps */
+#else
+	struct tls_data_container_t container;
+
+	/* Cached _stp_map_init() values. */
+	unsigned max_entries;
+	int wrap;
+	int type;
+	int key_size;
+	int data_size;
+#endif
 	struct map_root agg;	/* aggregation map */
 };
 typedef struct pmap *PMAP;
@@ -174,9 +196,9 @@ static int int64_eq_p(int64_t key1, int64_t key2);
 void int64_copy(void *dest, int64_t val);
 void int64_add(void *dest, int64_t val);
 int64_t int64_get(void *ptr);
-void stat_copy(void *dest, stat *src);
-void stat_add(void *dest, stat *src);
-stat *stat_get(void *ptr);
+void stat_copy(void *dest, stat_data *src);
+void stat_add(void *dest, stat_data *src);
+stat_data *stat_get(void *ptr);
 static int64_t _stp_key_get_int64(struct map_node *mn, int n);
 static char * _stp_key_get_str(struct map_node *mn, int n);
 static unsigned int int64_hash(const int64_t v);
@@ -186,14 +208,17 @@ static void str_add(void *dest, char *val);
 static int str_eq_p(char *key1, char *key2);
 static int64_t _stp_get_int64(struct map_node *m);
 static char * _stp_get_str(struct map_node *m);
-static stat *_stp_get_stat(struct map_node *m);
+static stat_data *_stp_get_stat(struct map_node *m);
 static unsigned int str_hash(const char *key1);
-static MAP _stp_map_new(unsigned max_entries, int type, int key_size, int data_size);
-static PMAP _stp_pmap_new(unsigned max_entries, int type, int key_size, int data_size);
+static MAP _stp_map_new(unsigned max_entries, int wrap, int type, int key_size,
+			int data_size);
+static PMAP _stp_pmap_new(unsigned max_entries, int wrap, int type,
+			  int key_size, int data_size);
 static int msb64(int64_t x);
-static MAP _stp_map_new_hstat_log(unsigned max_entries, int key_size);
-static MAP _stp_map_new_hstat_linear(unsigned max_entries, int ksize, int start, int stop, int interval);
-static void _stp_map_print_histogram(MAP map, stat *s);
+static MAP _stp_map_new_hstat_log(unsigned max_entries, int wrap, int key_size);
+static MAP _stp_map_new_hstat_linear(unsigned max_entries, int wrap, int ksize,
+				     int start, int stop, int interval);
+static void _stp_map_print_histogram(MAP map, stat_data *s);
 static struct map_node * _stp_map_start(MAP map);
 static struct map_node * _stp_map_iter(MAP map, struct map_node *m);
 static void _stp_map_del(MAP map);
@@ -205,8 +230,11 @@ static int _new_map_set_int64 (MAP map, struct map_node *n, int64_t val, int add
 static int _new_map_set_str (MAP map, struct map_node *n, char *val, int add);
 static void _new_map_clear_node (struct map_node *);
 static void _new_map_del_node (MAP map, struct map_node *n);
-static PMAP _stp_pmap_new_hstat_linear (unsigned max_entries, int ksize, int start, int stop, int interval);
-static PMAP _stp_pmap_new_hstat_log (unsigned max_entries, int key_size);
+static PMAP _stp_pmap_new_hstat_linear (unsigned max_entries, int wrap,
+					int ksize, int start, int stop,
+					int interval);
+static PMAP _stp_pmap_new_hstat_log (unsigned max_entries, int wrap,
+				     int key_size);
 static void _stp_add_agg(struct map_node *aptr, struct map_node *ptr);
 static struct map_node *_stp_new_agg(MAP agg, struct hlist_head *ahead, struct map_node *ptr);
 static void __stp_map_del(MAP map);
