@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * pmap API generator
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2008, 2012 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -58,13 +58,13 @@
 #define MAP_GET_VAL(n) _stp_get_int64(n)
 #define NULLRET (int64_t)0
 #elif VALUE_TYPE == STAT
-#define VALTYPE stat*
+#define VALTYPE stat_data*
 #define VSTYPE int64_t
-#define VALNAME stat
+#define VALNAME stat_data
 #define VALN x
 #define MAP_SET_VAL(a,b,c,d) _new_map_set_stat(a,b,c,d)
 #define MAP_GET_VAL(n) _stp_get_stat(n)
-#define NULLRET (stat*)0
+#define NULLRET (stat_data*)0
 #else
 #error Need to define VALUE_TYPE as STRING, STAT, or INT64
 #endif /* VALUE_TYPE */
@@ -653,15 +653,43 @@ static unsigned int KEYSYM(phash) (ALLKEYSD(key))
 	return (unsigned int) (hash % HASH_TABLE_SIZE);
 }
 
+#ifndef __KERNEL__
+static int _stp_map_tls_object_init(struct tls_data_object_t *obj);
+static void _stp_map_tls_object_free(struct tls_data_object_t *obj);
+
+static int KEYSYM(_stp_pmap_tls_object_init)(struct tls_data_object_t *obj)
+{
+	MAP m = container_of(obj, struct map_root, object);
+	PMAP p = container_of(obj->container, struct pmap, container);
+
+	if (_stp_map_tls_object_init(obj) != 0)
+		return -1;
+
+	/* Copy the hist params from the agg. We need to do this in
+	   case this pmap contains an hstat. */
+	m->hist.type = p->agg.hist.type;
+	m->hist.start = p->agg.hist.start;
+	m->hist.stop = p->agg.hist.stop;
+	m->hist.interval = p->agg.hist.interval;
+	m->hist.buckets = p->agg.hist.buckets;
+
+	m->get_key = KEYSYM(pmap_get_key);
+	m->copy = KEYSYM(pmap_copy_keys);
+	m->cmp = KEYSYM(pmap_key_cmp);
+	return 0;
+}
+#endif
 
 #if VALUE_TYPE == INT64 || VALUE_TYPE == STRING
-static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries)
+static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int wrap)
 {
-	PMAP pmap = _stp_pmap_new (max_entries, VALUE_TYPE, sizeof(struct KEYSYM(pmap_node)), 0);
+	PMAP pmap = _stp_pmap_new (max_entries, wrap, VALUE_TYPE,
+				   sizeof(struct KEYSYM(pmap_node)), 0);
 	if (pmap) {
 		int i;
 		MAP m;
-		stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+		for_each_possible_cpu(i) {
 			m = (MAP)per_cpu_ptr (pmap->map, i);
 			m->get_key = KEYSYM(pmap_get_key);
 			m->copy = KEYSYM(pmap_copy_keys);
@@ -670,6 +698,13 @@ static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries)
 			spin_lock_init(m->lock);
 #endif
 		}
+#else
+		/* Override the tls data object init function with one
+		 * that knows how to handle pmaps. */
+		_stp_tls_data_container_update(&pmap->container,
+					       &KEYSYM(_stp_pmap_tls_object_init),
+					       &_stp_map_tls_object_free);
+#endif
 		m = &pmap->agg;
 		m->get_key = KEYSYM(pmap_get_key);
 		m->copy = KEYSYM(pmap_copy_keys);
@@ -678,17 +713,19 @@ static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries)
 	return pmap;
 }
 #else
-/* _stp_pmap_new_key1_key2...val (num, HIST_LINEAR, start, end, interval) */
-/* _stp_pmap_new_key1_key2...val (num, HIST_LOG) */ 
-
-static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
+/*
+ * _stp_pmap_new_key1_key2...val (num, wrap, HIST_LINEAR, start, end, interval) 
+ * _stp_pmap_new_key1_key2...val (num, wrap, HIST_LOG)
+ */
+static PMAP
+KEYSYM(_stp_pmap_new) (unsigned max_entries, int wrap, int htype, ...)
 {
 	int start=0, stop=0, interval=0;
 	PMAP pmap;
 
 	if (htype == HIST_LINEAR) {
 		va_list ap;
-		va_start (ap, htype);		
+		va_start (ap, htype);
 		start = va_arg(ap, int);
 		stop = va_arg(ap, int);
 		interval = va_arg(ap, int);
@@ -697,14 +734,17 @@ static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 
 	switch (htype) {
 	case HIST_NONE:
-		pmap = _stp_pmap_new (max_entries, STAT, sizeof(struct KEYSYM(pmap_node)), 0);
+		pmap = _stp_pmap_new (max_entries, wrap, STAT,
+				      sizeof(struct KEYSYM(pmap_node)), 0);
 		break;
 	case HIST_LOG:
-		pmap = _stp_pmap_new_hstat_log (max_entries, sizeof(struct KEYSYM(pmap_node)));
+		pmap = _stp_pmap_new_hstat_log (max_entries, wrap,
+						sizeof(struct KEYSYM(pmap_node)));
 		break;
 	case HIST_LINEAR:
-		pmap = _stp_pmap_new_hstat_linear (max_entries, sizeof(struct KEYSYM(pmap_node)),
-					       start, stop, interval);
+		pmap = _stp_pmap_new_hstat_linear (max_entries, wrap,
+						   sizeof(struct KEYSYM(pmap_node)),
+						   start, stop, interval);
 		break;
 	default:
 		_stp_warn ("Unknown histogram type %d\n", htype);
@@ -714,7 +754,8 @@ static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 	if (pmap) {
 		int i;
 		MAP m;
-		stp_for_each_cpu(i) {
+#ifdef __KERNEL__
+		for_each_possible_cpu(i) {
 			m = per_cpu_ptr (pmap->map, i);
 			m->get_key = KEYSYM(pmap_get_key);
 			m->copy = KEYSYM(pmap_copy_keys);
@@ -723,6 +764,13 @@ static PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 			spin_lock_init(m->lock);
 #endif
 		}
+#else
+		/* Override the tls data object init function with one
+		 * that knows how to handle pmaps. */
+		_stp_tls_data_container_update(&pmap->container,
+					       &KEYSYM(_stp_pmap_tls_object_init),
+					       &_stp_map_tls_object_free);
+#endif
 		m = &pmap->agg;
 		m->get_key = KEYSYM(pmap_get_key);
 		m->copy = KEYSYM(pmap_copy_keys);
@@ -792,7 +840,17 @@ static int KEYSYM(__stp_pmap_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 static int KEYSYM(_stp_pmap_set) (PMAP pmap, ALLKEYSD(key), VSTYPE val)
 {
 	int res;
+#ifdef __KERNEL__
 	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#else
+	struct tls_data_object_t *obj;
+	MAP m;
+
+	obj = _stp_tls_get_per_thread_ptr(&pmap->container);
+	if (!obj)
+		return -ENOMEM;
+	m = container_of(obj, struct map_root, object);
+#endif
 #if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -3;
@@ -808,7 +866,17 @@ static int KEYSYM(_stp_pmap_set) (PMAP pmap, ALLKEYSD(key), VSTYPE val)
 static int KEYSYM(_stp_pmap_add) (PMAP pmap, ALLKEYSD(key), VSTYPE val)
 {
 	int res;
-	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU());
+#ifdef __KERNEL__
+	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#else
+	struct tls_data_object_t *obj;
+	MAP m;
+
+	obj = _stp_tls_get_per_thread_ptr(&pmap->container);
+	if (!obj)
+		return -ENOMEM;
+	m = container_of(obj, struct map_root, object);
+#endif
 #if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -3;
@@ -830,11 +898,21 @@ static VALTYPE KEYSYM(_stp_pmap_get_cpu) (PMAP pmap, ALLKEYSD(key))
 	struct KEYSYM(pmap_node) *n;
 	VALTYPE res;
 	MAP map;
+#ifndef __KERNEL__
+	struct tls_data_object_t *obj;
+#endif
 
 	if (pmap == NULL)
 		return NULLRET;
 
+#ifdef __KERNEL__
 	map = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#else
+	obj = _stp_tls_get_per_thread_ptr(&pmap->container);
+	if (!obj)
+		return NULLRET;
+	map = container_of(obj, struct map_root, object);
+#endif
 
 	hv = KEYSYM(phash) (ALLKEYS(key));
 	head = &map->hashes[hv];
@@ -896,6 +974,9 @@ static VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 	struct KEYSYM(pmap_node) *n;
 	struct map_node *anode = NULL;
 	MAP map, agg;
+#ifndef __KERNEL__
+	struct tls_data_object_t *obj;
+#endif
 
 	if (pmap == NULL)
 		return NULLRET;
@@ -940,13 +1021,23 @@ static VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 	}
 
 	/* now total each cpu */
-	stp_for_each_cpu(cpu) {
+#ifdef __KERNEL__
+	for_each_possible_cpu(cpu) {
 		map = per_cpu_ptr (pmap->map, cpu);
+#else
+	TLS_DATA_CONTAINER_LOCK(&pmap->container);
+	for_each_tls_data(obj, &pmap->container) {
+		map = container_of(obj, struct map_root, object);
+#endif
 		head = &map->hashes[hv];
 
 #if NEED_MAP_LOCKS
-		if (!spin_trylock(&map->lock))
+		if (!spin_trylock(&map->lock)) {
+#ifndef __KERNEL__
+			TLS_DATA_CONTAINER_UNLOCK(&pmap->container);
+#endif
 			return NULLRET;
+		}
 #endif
 
 		hlist_for_each(e, head) {
@@ -992,6 +1083,9 @@ static VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 		spin_unlock(&map->lock);
 #endif
 	}
+#ifndef __KERNEL__
+	TLS_DATA_CONTAINER_UNLOCK(&pmap->container);
+#endif
 	if (anode && !clear_agg) 
 		return MAP_GET_VAL(anode);
 
@@ -1055,7 +1149,17 @@ static int KEYSYM(__stp_pmap_del) (MAP map, ALLKEYSD(key))
 static int KEYSYM(_stp_pmap_del) (PMAP pmap, ALLKEYSD(key))
 {
 	int res;
+#ifdef __KERNEL__
 	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#else
+	struct tls_data_object_t *obj;
+	MAP m;
+
+	obj = _stp_tls_get_per_thread_ptr(&pmap->container);
+	if (!obj)
+		return -ENOMEM;
+	m = container_of(obj, struct map_root, object);
+#endif
 #if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -1;
