@@ -1,6 +1,6 @@
 /* -*- linux-c -*-
  * Statistics Aggregation
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2008, 2012 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation
  *
  * This file is part of systemtap, and is free software.  You can
@@ -38,28 +38,67 @@
  * @{
  */
 
+#ifndef __KERNEL__
+#include <pthread.h>
+#endif
+
 #include "stat-common.c"
 
 /* for the paranoid. */
 #if NEED_STAT_LOCKS == 1
-#define STAT_LOCK(st) spin_lock(&st->lock)
-#define STAT_UNLOCK(st) spin_unlock(&st->lock)
+#ifdef __KERNEL__
+#define STAT_LOCK(sd) spin_lock(&sd->lock)
+#define STAT_UNLOCK(sd) spin_unlock(&sd->lock)
 #else
-#define STAT_LOCK(st) ;
-#define STAT_UNLOCK(st) ;
+#define STAT_LOCK(sd) pthread_mutex_lock(&sd->lock)
+#define STAT_UNLOCK(sd) pthread_mutex_unlock(&sd->lock)
+#endif
+#else
+#define STAT_LOCK(sd) ;
+#define STAT_UNLOCK(sd) ;
 #endif
 
 /** Stat struct for stat.c. Maps do not need this */
 struct _Stat {
 	struct _Hist hist;
-	/* per-cpu data. allocated with _stp_alloc_percpu() */
-	stat *sd;
+
+	/*
+	 * In kernel-mode, the stat data is per-cpu data (allocated
+	 * with _stp_alloc_percpu()) stored in 'sd'. In dyninst-mode,
+	 * the stat data is thread local storage.
+	 */
+#ifdef __KERNEL__
+	stat_data *sd;
+#else
+	struct tls_data_container_t container;
+#endif
 	/* aggregated data */   
-	stat *agg;  
+	stat_data *agg;  
 };
 
 typedef struct _Stat *Stat;
 
+#ifndef __KERNEL__
+#if NEED_STAT_LOCKS == 1
+static int _stp_stat_tls_object_init(struct tls_data_object_t *obj)
+{
+	stat_data *sd = container_of(obj, stat_data, object);
+
+	int rc;
+	if ((rc = pthread_mutex_init(&sd->lock, NULL)) != 0) {
+		_stp_error("Couldn't initialize stat mutex: %d\n", rc);
+		return -1;
+	}
+	return 0;
+}
+
+static void _stp_stat_tls_object_free(struct tls_data_object_t *obj)
+{
+	stat_data *sd = container_of(obj, stat_data, object);
+	(void)pthread_mutex_destroy(&sd->lock);
+}
+#endif	/* NEED_STAT_LOCKS == 1 */
+#endif	/* !__KERNEL__ */
 
 /** Initialize a Stat.
  * Call this during probe initialization to create a Stat.
@@ -77,7 +116,11 @@ typedef struct _Stat *Stat;
 static Stat _stp_stat_init (int type, ...)
 {
 	int size, buckets=0, start=0, stop=0, interval=0;
-	stat *sd, *agg;
+#ifdef __KERNEL__
+	stat_data *sd, *agg;
+#else
+	stat_data *agg;
+#endif
 	Stat st;
 
 	if (type != HIST_NONE) {
@@ -102,22 +145,37 @@ static Stat _stp_stat_init (int type, ...)
 	if (st == NULL)
 		return NULL;
 	
-	size = buckets * sizeof(int64_t) + sizeof(stat);	
-	sd = (stat *) _stp_alloc_percpu (size);
+	size = buckets * sizeof(int64_t) + sizeof(stat_data);	
+#ifdef __KERNEL__
+	sd = (stat_data *) _stp_alloc_percpu (size);
 	if (sd == NULL)
 		goto exit1;
+	st->sd = sd;
 
 #if NEED_STAT_LOCKS == 1
 	{
 		int i;
-		stp_for_each_cpu(i) {
-			stat *sdp = per_cpu_ptr (sd, i);
+		for_each_possible_cpu(i) {
+			stat_data *sdp = per_cpu_ptr (sd, i);
 			spin_lock_init(sdp->lock);
 		}
 	}
-#endif
+#endif	/* NEED_STAT_LOCKS == 1 */
+
+#else  /* !__KERNEL__ */
+
+#if NEED_STAT_LOCKS == 1
+	if (_stp_tls_data_container_init(&st->container, size,
+					 &_stp_stat_tls_object_init,
+					 &_stp_stat_tls_object_free) != 0)
+#else  /* NEED_STAT_LOCKS !=1 */
+	if (_stp_tls_data_container_init(&st->container, size,
+					 NULL, NULL) != 0)
+#endif	/* NEED_STAT_LOCKS != 1 */
+		goto exit1;
+#endif	/* !__KERNEL__ */
 	
-	agg = (stat *)_stp_kmalloc_gfp(size, STP_ALLOC_SLEEP_FLAGS);
+	agg = (stat_data *)_stp_kmalloc_gfp(size, STP_ALLOC_SLEEP_FLAGS);
 	if (agg == NULL)
 		goto exit2;
 
@@ -126,12 +184,15 @@ static Stat _stp_stat_init (int type, ...)
 	st->hist.stop = stop;
 	st->hist.interval = interval;
 	st->hist.buckets = buckets;
-	st->sd = sd;
 	st->agg = agg;
 	return st;
 
 exit2:
+#ifdef __KERNEL__
 	_stp_kfree (sd);
+#else
+	_stp_tls_data_container_cleanup(&st->container);
+#endif
 exit1:
 	_stp_kfree (st);
 	return NULL;
@@ -145,7 +206,11 @@ exit1:
 static void _stp_stat_del (Stat st)
 {
 	if (st) {
+#ifdef __KERNEL__
 		_stp_free_percpu (st->sd);
+#else  /* !__KERNEL__ */
+		_stp_tls_data_container_cleanup(&st->container);
+#endif /* !__KERNEL__ */
 		_stp_kfree (st->agg);
 		_stp_kfree (st);
 	}
@@ -159,31 +224,25 @@ static void _stp_stat_del (Stat st)
  */
 static void _stp_stat_add (Stat st, int64_t val)
 {
-	stat *sd = per_cpu_ptr (st->sd, get_cpu());
+#ifdef __KERNEL__
+	stat_data *sd = per_cpu_ptr (st->sd, get_cpu());
+#else
+	struct tls_data_object_t *obj;
+	stat_data *sd;
+
+	obj = _stp_tls_get_per_thread_ptr(&st->container);
+	if (!obj)
+		return;
+	sd = container_of(obj, stat_data, object);
+#endif
 	STAT_LOCK(sd);
 	__stp_stat_add (&st->hist, sd, val);
 	STAT_UNLOCK(sd);
 	put_cpu();
 }
 
-/** Get per-cpu Stats.
- * Gets the Stats for a specific CPU.
- *
- * If NEED_STAT_LOCKS is set, you MUST call STAT_UNLOCK()
- * when you are finished with the returned pointer.
- *
- * @param st Stat
- * @param cpu CPU number
- * @returns A pointer to a stat.
- */
-static stat *_stp_stat_get_cpu (Stat st, int cpu)
-{
-	stat *sd = per_cpu_ptr (st->sd, cpu);
-	STAT_LOCK(sd);
-	return sd;
-}
 
-static void _stp_stat_clear_data (Stat st, stat *sd)
+static void _stp_stat_clear_data (Stat st, stat_data *sd)
 {
         int j;
         sd->count = sd->sum = sd->min = sd->max = 0;
@@ -204,15 +263,25 @@ static void _stp_stat_clear_data (Stat st, stat *sd)
  * for polling.
  * @returns A pointer to a stat.
  */
-static stat *_stp_stat_get (Stat st, int clear)
+static stat_data *_stp_stat_get (Stat st, int clear)
 {
 	int i, j;
-	stat *agg = st->agg;
+	stat_data *agg = st->agg;
+	stat_data *sd;
+#ifndef __KERNEL__
+	struct tls_data_object_t *obj;
+#endif
 	STAT_LOCK(agg);
 	_stp_stat_clear_data (st, agg);
 
-	stp_for_each_cpu(i) {
-		stat *sd = per_cpu_ptr (st->sd, i);
+#ifdef __KERNEL__
+	for_each_possible_cpu(i) {
+		sd = per_cpu_ptr (st->sd, i);
+#else
+	TLS_DATA_CONTAINER_LOCK(&st->container);
+	for_each_tls_data(obj, &st->container) {
+		sd = container_of(obj, stat_data, object);
+#endif
 		STAT_LOCK(sd);
 		if (sd->count) {
 			if (agg->count == 0) {
@@ -234,6 +303,9 @@ static stat *_stp_stat_get (Stat st, int clear)
 		}
 		STAT_UNLOCK(sd);
 	}
+#ifndef __KERNEL__
+	TLS_DATA_CONTAINER_UNLOCK(&st->container);
+#endif
 	return agg;
 }
 
@@ -245,13 +317,23 @@ static stat *_stp_stat_get (Stat st, int clear)
  */
 static void _stp_stat_clear (Stat st)
 {
+#ifdef __KERNEL__
 	int i;
-	stp_for_each_cpu(i) {
-		stat *sd = per_cpu_ptr (st->sd, i);
+	for_each_possible_cpu(i) {
+		stat_data *sd = per_cpu_ptr (st->sd, i);
+#else
+	struct tls_data_object_t *obj;
+	TLS_DATA_CONTAINER_LOCK(&st->container);
+	for_each_tls_data(obj, &st->container) {
+		stat_data *sd = container_of(obj, stat_data, object);
+#endif
 		STAT_LOCK(sd);
 		_stp_stat_clear_data (st, sd);
 		STAT_UNLOCK(sd);
 	}
+#ifndef __KERNEL__
+	TLS_DATA_CONTAINER_UNLOCK(&st->container);
+#endif
 }
 /** @} */
 #endif /* _STAT_C_ */
