@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Statistics Header
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005, 2012 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -11,6 +11,10 @@
 #ifndef _STAT_H_
 #define _STAT_H_
 
+#ifndef __KERNEL__
+#include "dyninst/tls_data.c"
+#endif
+
 #ifndef NEED_STAT_LOCKS
 #define NEED_STAT_LOCKS 0
 #endif
@@ -30,15 +34,24 @@ enum histtype { HIST_NONE, HIST_LOG, HIST_LINEAR };
 /** Statistics are stored in this struct.  This is per-cpu or per-node data 
     and is variable length due to the unknown size of the histogram. */
 struct stat_data {
+#ifndef __KERNEL__
+	/* Note that the tls_data_object_t must be first in struct
+	 * stat_data. */
+	struct tls_data_object_t object;
+#endif
 	int64_t count;
 	int64_t sum;
 	int64_t min, max;
 #if NEED_STAT_LOCKS == 1
+#ifdef __KERNEL__
 	spinlock_t lock;
+#else  /* !__KERNEL__ */
+	pthread_mutex_t lock;
+#endif	/* !__KERNEL__ */
 #endif
 	int64_t histogram[];
 };
-typedef struct stat_data stat;
+typedef struct stat_data stat_data;
 
 /** Information about the histogram data collected. This data 
     is global and not duplicated per-cpu. */
