@@ -14,7 +14,7 @@
 
 #include "sym.h"
 #include "vma.c"
-#include "string.c"
+#include "stp_string.c"
 #include <asm/uaccess.h>
 
 #ifdef STAPCONF_PROBE_KERNEL
@@ -239,32 +239,54 @@ static const char *_stp_kallsyms_lookup(unsigned long addr,
 	return NULL;
 }
 
-static int _stp_build_id_check (struct _stp_module *m, unsigned long notes_addr, int user_module)
+static int _stp_build_id_check (struct _stp_module *m,
+				unsigned long notes_addr,
+				struct task_struct *tsk)
 {
   int j;
-  for (j=0; j<m->build_id_len; j++) {
-    /* Use set_fs / get_user to access
-       conceivably invalid addresses.  If
-       loc2c-runtime.h were more easily usable,
-       a deref() loop could do it too. */
+
+  for (j = 0; j < m->build_id_len; j++) {
+    /* Use set_fs / get_user to access conceivably invalid addresses.
+     * If loc2c-runtime.h were more easily usable, a deref() loop
+     * could do it too. */
     mm_segment_t oldfs = get_fs();
     int rc;
-    unsigned char theory, practice;
+    unsigned char theory, practice = 0;
 
 #ifdef STAPCONF_PROBE_KERNEL
-    if (!user_module)
-      {
-        theory = m->build_id_bits[j];
-        rc=probe_kernel_read(&practice, (void*)(notes_addr+j), 1);
-      }
+    if (!tsk) {
+      theory = m->build_id_bits[j];
+      set_fs(KERNEL_DS);
+      rc = probe_kernel_read(&practice, (void*)(notes_addr + j), 1);
+    }
     else
 #endif
-      {
-        set_fs (user_module ? USER_DS : KERNEL_DS);
-        theory = m->build_id_bits[j];
-        rc = get_user(practice,((unsigned char*) (void*) (notes_addr+j)));
-        set_fs(oldfs);
+    {
+      theory = m->build_id_bits[j];
+      set_fs (tsk ? USER_DS : KERNEL_DS);
+
+      /*
+       * Why check CONFIG_UTRACE here? If we're using real in-kernel
+       * utrace, we can always just call get_user() (since we're
+       * either reading kernel memory or tsk == current).
+       *
+       * Since we're only reading here, we can call
+       * __access_process_vm_noflush(), which only calls things that
+       * are exported.
+       */
+#ifdef CONFIG_UTRACE
+      rc = get_user(practice, ((unsigned char*)(void*)(notes_addr + j)));
+#else
+      if (!tsk || tsk == current) {
+	rc = get_user(practice, ((unsigned char*)(void*)(notes_addr + j)));
       }
+      else {
+	rc = (__access_process_vm_noflush(tsk, (notes_addr + j), &practice,
+					  1, 0) != 1);
+      }
+#endif
+    }
+    set_fs(oldfs);
 
     if (rc || (theory != practice)) {
       const char *basename;
@@ -279,8 +301,8 @@ static int _stp_build_id_check (struct _stp_module *m, unsigned long notes_addr,
 		  m->name, basename, j, theory, practice, notes_addr, rc);
       return 1;
 #else
-      /* This branch is a surrogate for kernels
-       * affected by Fedora bug #465873. */
+      /* This branch is a surrogate for kernels affected by Fedora bug
+       * #465873. */
       _stp_warn (KERN_WARNING
 		 "Build-id mismatch: \"%s\" vs. \"%s\" byte %d (0x%02x vs 0x%02x) rc %d\n",
 		 m->name, basename, j, theory, practice, rc);
@@ -329,11 +351,11 @@ static int _stp_module_check(void)
           }
 
           if (notes_addr <= base_addr) { /* shouldn't happen */
-              _stp_warn ("build-id address %lx < base %lx\n",
+              _stp_warn ("build-id address %lx <= base %lx\n",
                   notes_addr, base_addr);
               continue;
           }
-          return _stp_build_id_check (m, notes_addr, 0);
+          return _stp_build_id_check (m, notes_addr, NULL);
       } /* end checking */
     } /* end loop */
   return 0;
@@ -370,7 +392,7 @@ static int _stp_kmodule_check (const char *name)
                   notes_addr, base_addr);
               continue;
           }
-          return _stp_build_id_check (m, notes_addr, 0);
+          return _stp_build_id_check (m, notes_addr, NULL);
       } /* end checking */
     } /* end loop */
 
@@ -410,7 +432,7 @@ static int _stp_usermodule_check(struct task_struct *tsk, const char *path_name,
 		     notes_addr, addr);
 	  continue;
 	}
-	return _stp_build_id_check (m, notes_addr, 1);
+	return _stp_build_id_check (m, notes_addr, tsk);
       }
     }
 
@@ -622,5 +644,14 @@ static void _stp_kmodule_update_address(const char* module,
 }
 
 
+#ifndef STAPCONF_KALLSYMS
+unsigned long kallsyms_lookup_name (const char *name)
+{
+        _stp_error ("kallsyms_lookup_name unavailable for %s\n", name);
+        return 0;
+}
+#endif
+
+
 
 #endif /* _STP_SYM_C_ */
