@@ -20,6 +20,7 @@
 #include <sstream>
 #include <map>
 #include <set>
+#include <stdexcept>
 
 extern "C" {
 #include <signal.h>
@@ -27,23 +28,11 @@ extern "C" {
 }
 
 #include "privilege.h"
-
-#if ENABLE_NLS
-#define _(string) gettext(string)
-#define _N(string, string_plural, count) \
-        ngettext((string), (string_plural), (count))
-#else
-#define _(string) (string)
-#define _N(string, string_plural, count) \
-        ( (count) == 1 ? (string) : (string_plural) )
-#endif
-#define _F(format, ...) autosprintf(_(format), __VA_ARGS__)
-#define _NF(format, format_plural, count, ...) \
-        autosprintf(_N((format), (format_plural), (count)), __VA_ARGS__)
+#include "util.h"
 
 // forward decls for all referenced systemtap types
-struct hash;
-struct match_node;
+class stap_hash;
+class match_node;
 struct stapfile;
 struct vardecl;
 struct token;
@@ -59,13 +48,14 @@ struct utrace_derived_probe_group;
 struct itrace_derived_probe_group;
 struct task_finder_derived_probe_group;
 struct timer_derived_probe_group;
+struct netfilter_derived_probe_group;
 struct profile_derived_probe_group;
 struct mark_derived_probe_group;
 struct tracepoint_derived_probe_group;
 struct hrtimer_derived_probe_group;
 struct procfs_derived_probe_group;
 struct embeddedcode;
-struct translator_output;
+class translator_output;
 struct unparser;
 struct semantic_error;
 struct module_cache;
@@ -93,6 +83,8 @@ struct statistic_decl
   }
 };
 
+struct macrodecl; // defined in parse.h
+
 struct systemtap_session
 {
 private:
@@ -109,6 +101,11 @@ public:
   systemtap_session ();
   ~systemtap_session ();
 
+  // To reset the tmp_dir
+  void create_tmp_dir();
+  void remove_tmp_dir();
+  void reset_tmp_dir();
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
   void setup_kernel_release (const char* kstr);
@@ -130,7 +127,7 @@ public:
   bool have_script;
   std::vector<std::string> include_path;
   int include_arg_start;
-  std::vector<std::string> macros;
+  std::vector<std::string> c_macros;
   std::vector<std::string> args;
   std::vector<std::string> kbuildflags; // -B var=val
   std::vector<std::string> globalopts; // -G var=val
@@ -140,6 +137,9 @@ public:
   std::string kernel_base_release;
   std::string kernel_build_tree;
   std::string kernel_source_tree;
+  std::string sysroot;
+  std::map<std::string,std::string> sysenv;
+  bool update_release_sysroot;
   std::map<std::string,std::string> kernel_config;
   std::set<std::string> kernel_exports;
   std::string machine;
@@ -149,6 +149,7 @@ public:
   bool runtime_specified;
   std::string data_path;
   std::string module_name;
+  const std::string module_filename() const;
   std::string stapconf_name;
   std::string output_file;
   std::string size_option;
@@ -187,6 +188,9 @@ public:
   int download_dbinfo;
   bool suppress_handler_errors;
 
+  enum { kernel_runtime, dyninst_runtime } runtime_mode;
+  bool runtime_usermode_p() const { return runtime_mode == dyninst_runtime; }
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 
@@ -196,7 +200,7 @@ public:
   void NSPR_init ();
 #endif
   bool client_options;
-  std::string client_options_disallowed;
+  std::string client_options_disallowed_for_unprivileged;
   std::vector<std::string> server_status_strings;
   std::vector<std::string> specified_servers;
   bool automatic_server_mode;
@@ -240,7 +244,7 @@ public:
   std::string cache_path;       // usually ~/.systemtap/cache
   std::string hash_path;        // path to the cached script module
   std::string stapconf_path;    // path to the cached stapconf
-  hash *base_hash;              // hash common to all caching
+  stap_hash *base_hash;         // hash common to all caching
 
   // dwarfless operation
   bool consult_symtab;
@@ -262,6 +266,10 @@ public:
   match_node* pattern_root;
   void register_library_aliases();
 
+  // data for various preprocessor library macros
+  std::map<std::string, macrodecl*> library_macros;
+  std::vector<stapfile*> library_macro_files; // for error reporting purposes
+
   // parse trees for the various script files
   stapfile* user_file;
   std::vector<stapfile*> library_files;
@@ -295,6 +303,7 @@ public:
   itrace_derived_probe_group* itrace_derived_probes;
   task_finder_derived_probe_group* task_finder_derived_probes;
   timer_derived_probe_group* timer_derived_probes;
+  netfilter_derived_probe_group* netfilter_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
   mark_derived_probe_group* mark_derived_probes;
   tracepoint_derived_probe_group* tracepoint_derived_probes;
@@ -345,9 +354,27 @@ public:
 };
 
 
+struct exit_exception: public std::runtime_error
+{
+  int rc;
+  exit_exception (int rc):
+    runtime_error (_F("early exit requested, rc=%d", rc)), rc(rc) {}
+};
+
+
 // global counter of SIGINT/SIGTERM's received
 extern int pending_interrupts;
 
+// Interrupt exception subclass for catching
+// interrupts (i.e. ctrl-c).
+struct interrupt_exception: public std::runtime_error
+{
+  interrupt_exception ():
+    runtime_error (_("interrupt received")){}
+};
+
+void assert_no_interrupts();
+
 #endif // SESSION_H
 
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
