@@ -146,7 +146,7 @@ probe_point::component::component (std::string const & f, literal * a):
 
 
 vardecl::vardecl ():
-  arity_tok(0), arity (-1), maxsize(0), init(NULL), skip_init(false), wrap(false)
+  arity_tok(0), arity (-1), maxsize(0), init(NULL), synthetic(false), wrap(false)
 {
 }
 
@@ -192,7 +192,7 @@ vardecl::compatible_arity (int a)
 
 
 functiondecl::functiondecl ():
-  body (0), synthetic (false)
+  body (0), synthetic (false), mangle_oldstyle (false)
 {
 }
 
@@ -275,6 +275,18 @@ void target_symbol::chain (const semantic_error &er)
   this->saved_conversion_error = e;
 }
 
+string target_symbol::sym_name ()
+{
+  if (name == "@var")
+    {
+      if (cu_name == "")
+	return target_name;
+      else
+	return target_name.substr(0, target_name.length() - cu_name.length() - 1);
+    }
+  else
+    return name.substr(1);
+}
 
 // ------------------------------------------------------------------------
 // parse tree printing
@@ -389,6 +401,8 @@ void target_symbol::print (ostream& o) const
   if (addressof)
     o << "&";
   o << name;
+  if (name == "@var")
+    o << "(\"" << target_name << "\")";
   for (unsigned i = 0; i < components.size(); ++i)
     o << components[i];
 }
@@ -1928,8 +1942,10 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 
   // Don't allow embedded C functions in unprivileged mode unless
   // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
+  // or we're in a usermode runtime.
   if (! pr_contains (session.privilege, pr_stapdev) &&
       ! pr_contains (session.privilege, pr_stapsys) &&
+      ! session.runtime_usermode_p () &&
       s->code.find ("/* unprivileged */") == string::npos &&
       s->code.find ("/* myproc-unprivileged */") == string::npos)
     throw semantic_error (_F("function may not be used when --privilege=%s is specified",
@@ -1941,6 +1957,10 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
     throw semantic_error (_("function may not be used unless -g is specified"),
 			  current_function->tok);
 
+  // PR14524: Support old-style THIS->local syntax on per-function basis.
+  if (s->code.find ("/* unmangled */") != string::npos)
+    current_function->mangle_oldstyle = true;
+
   // We want to elide embedded-C functions when possible.  For
   // example, each $target variable access is expanded to an
   // embedded-C function call.  Yet, for safety reasons, we should
@@ -1963,9 +1983,11 @@ void
 varuse_collecting_visitor::visit_embedded_expr (embedded_expr *e)
 {
   // Don't allow embedded C expressions in unprivileged mode unless
-  // they are tagged with /* unprivileged */
+  // they are tagged with /* unprivileged */ or /* myproc-unprivileged */
+  // or we're in a usermode runtime.
   if (! pr_contains (session.privilege, pr_stapdev) &&
       ! pr_contains (session.privilege, pr_stapsys) &&
+      ! session.runtime_usermode_p () &&
       e->code.find ("/* unprivileged */") == string::npos &&
       e->code.find ("/* myproc-unprivileged */") == string::npos)
     throw semantic_error (_F("embedded expression may not be used when --privilege=%s is specified",
