@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2011 Red Hat Inc.
+// Copyright (C) 2005-2012 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -28,16 +28,16 @@ struct systemtap_session; // session.h
 struct semantic_error: public std::runtime_error
 {
   const token* tok1;
-  std::string msg2;
   const token* tok2;
-  semantic_error *chain;
+  const semantic_error *chain;
 
   ~semantic_error () throw () {}
   semantic_error (const std::string& msg, const token* t1=0):
     runtime_error (msg), tok1 (t1), tok2 (0), chain (0) {}
+
   semantic_error (const std::string& msg, const token* t1,
-                  const std::string& m2, const token* t2):
-    runtime_error (msg), tok1 (t1), msg2 (m2), tok2 (t2), chain (0) {}
+                  const token* t2):
+                  runtime_error (msg), tok1 (t1), tok2 (t2), chain (0) {}
 };
 
 // ------------------------------------------------------------------------
@@ -215,7 +215,7 @@ classify_const_indexable(const indexable* ix,
 			 symbol const *& array_out,
 			 hist_op const *& hist_out);
 
-class vardecl;
+struct vardecl;
 struct symbol:
   public expression,
   public indexable
@@ -265,9 +265,12 @@ struct target_symbol: public symbol
     };
 
   bool addressof;
+  std::string target_name;
+  std::string cu_name;
   std::vector<component> components;
   semantic_error* saved_conversion_error; // hand-made linked list
   target_symbol(): addressof(false), saved_conversion_error (0) {}
+  std::string sym_name ();
   void chain (const semantic_error& er);
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -314,7 +317,7 @@ struct arrayindex: public expression
 };
 
 
-class functiondecl;
+struct functiondecl;
 struct functioncall: public expression
 {
   std::string function;
@@ -495,7 +498,7 @@ struct vardecl: public symboldecl
   int maxsize; // upperbound on size for arrays
   std::vector<exp_type> index_types; // for arrays only
   literal *init; // for global scalars only
-  bool skip_init; // for probe locals only, don't init on entry
+  bool synthetic; // for probe locals only, don't init on entry
   bool wrap;
 };
 
@@ -513,6 +516,7 @@ struct functiondecl: public symboldecl
   std::vector<vardecl*> unused_locals;
   statement* body;
   bool synthetic;
+  bool mangle_oldstyle;
   functiondecl ();
   void print (std::ostream& o) const;
   void printsig (std::ostream& o) const;
@@ -714,6 +718,7 @@ struct probe
   virtual void collect_derivation_chain (std::vector<probe*> &probes_list);
   virtual void collect_derivation_pp_chain (std::vector<probe_point*> &) {}
   virtual const probe_alias *get_alias () const { return 0; }
+  virtual probe_point *get_alias_loc () const { return 0; }
   virtual probe* create_alias(probe_point* l, probe_point* a);
   virtual const probe* basest () const { return this; }
   virtual const probe* almost_basest () const { return 0; }
