@@ -100,7 +100,7 @@ struct itrace_builder: public derived_probe_builder
 		     std::map<std::string, literal *> const & parameters,
 		     vector<derived_probe *> & finished_results)
   {
-    string path;
+    string path, path_tgt;
     int64_t pid = 0;
     int single_step;
 
@@ -114,12 +114,13 @@ struct itrace_builder: public derived_probe_builder
     // If we have a path, we need to validate it.
     if (has_path)
       {
-        path = find_executable (path);
+        path = find_executable (path, sess.sysroot, sess.sysenv);
         sess.unwindsym_modules.insert (path);
+        path_tgt = path_remove_sysroot(sess, path);
       }
 
     finished_results.push_back(new itrace_derived_probe(sess, base, location,
-							has_path, path, pid,
+							has_path, path_tgt, pid,
 							single_step
 							));
   }
@@ -183,16 +184,16 @@ itrace_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "int single_step;";
   s.op->newline(-1) << "};";
   s.op->newline() << "static void enter_itrace_probe(struct stap_itrace_probe *p, struct pt_regs *regs, void *data);";
-  s.op->newline() << "#include \"itrace.c\"";
+  s.op->newline() << "#include \"linux/itrace.c\"";
 
   // output routine to call itrace probe
   s.op->newline() << "static void enter_itrace_probe(struct stap_itrace_probe *p, struct pt_regs *regs, void *data) {";
   s.op->indent(1);
 
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "p->probe",
-				 "_STP_PROBE_HANDLER_ITRACE");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "p->probe",
+				 "stp_probe_type_itrace");
   s.op->newline() << "c->uregs = regs;";
-  s.op->newline() << "c->probe_flags |= _STP_PROBE_STATE_USER_MODE;";
+  s.op->newline() << "c->user_mode_p = 1;";
 
   // call probe function
   s.op->newline() << "(*p->probe->ph) (c);";
