@@ -233,7 +233,7 @@ mark_derived_probe::mark_derived_probe (systemtap_session &s,
 	v->tok = this->tok;
 	v->set_arity(0, this->tok);
 	v->type = mark_args[i]->stp_type;
-	v->skip_init = true;
+	v->synthetic = true;
 	this->locals.push_back (v);
       }
 
@@ -426,7 +426,8 @@ mark_derived_probe::initialize_probe_context_vars (translator_output* o)
   bool deref_fault_needed = false;
   for (unsigned i = 0; i < mark_args.size(); i++)
     {
-      string localname = "l->__mark_arg" + lex_cast(i+1);
+      string localname = "l->" +
+        sess.up->c_localname("__mark_arg" + lex_cast(i+1));
       switch (mark_args[i]->stp_type)
         {
 	case pe_long:
@@ -510,8 +511,8 @@ mark_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline();
   s.op->newline() << "static void enter_marker_probe (void *probe_data, void *call_data, const char *fmt, va_list *args) {";
   s.op->newline(1) << "struct stap_marker_probe *smp = (struct stap_marker_probe *)probe_data;";
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "smp->probe",
-				 "_STP_PROBE_HANDLER_MARKER");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "smp->probe",
+				 "stp_probe_type_marker");
   s.op->newline() << "c->ips.kmark.marker_name = smp->name;";
   s.op->newline() << "c->ips.kmark.marker_format = smp->format;";
   s.op->newline() << "c->ips.kmark.mark_va_list = args;";
