@@ -85,7 +85,7 @@ perf_derived_probe_group::emit_module_decls (systemtap_session& s)
   if (probes.empty()) return;
 
   s.op->newline() << "/* ---- perf probes ---- */";
-  s.op->newline() << "#include \"perf.c\"";
+  s.op->newline() << "#include \"linux/perf.c\"";
   s.op->newline();
 
   /* declarations */
@@ -147,10 +147,10 @@ perf_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "static void handle_perf_probe (unsigned i, struct pt_regs *regs)";
   s.op->newline() << "{";
   s.op->newline(1) << "struct stap_perf_probe* stp = & stap_perf_probes [i];";
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "stp->probe",
-				 "_STP_PROBE_HANDLER_PERF");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+				 "stp_probe_type_perf");
   s.op->newline() << "if (user_mode(regs)) {";
-  s.op->newline(1)<< "c->probe_flags |= _STP_PROBE_STATE_USER_MODE;";
+  s.op->newline(1)<< "c->user_mode_p = 1;";
   s.op->newline() << "c->uregs = regs;";
   s.op->newline(-1) << "} else {";
   s.op->newline(1) << "c->kregs = regs;";
@@ -175,8 +175,8 @@ perf_derived_probe_group::emit_module_init (systemtap_session& s)
   s.op->newline() << "for (j=0; j<i; j++) {";
   s.op->newline(1) << "_stp_perf_del(& stap_perf_probes [j]);";
   s.op->newline(-1) << "}"; // for unwind loop
-  s.op->newline(-1) << "}"; // if-error
   s.op->newline() << "break;";
+  s.op->newline(-1) << "}"; // if-error
   s.op->newline(-1) << "}"; // for loop
 }
 
