@@ -248,9 +248,9 @@ procfs_derived_probe_group::emit_module_decls (systemtap_session& s)
     {
       s.op->newline() << "struct _stp_procfs_data pdata;";
 
-      common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING",
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING",
 				     "spp->read_probe",
-				     "_STP_PROBE_HANDLER_PROCFS");
+				     "stp_probe_type_procfs");
 
       s.op->newline() << "pdata.buffer = spp->buffer;";
       s.op->newline() << "pdata.bufsize = spp->bufsize;";
@@ -294,9 +294,9 @@ procfs_derived_probe_group::emit_module_decls (systemtap_session& s)
     {
       s.op->newline() << "struct _stp_procfs_data pdata;";
 
-      common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING",
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING",
 				     "spp->write_probe",
-				     "_STP_PROBE_HANDLER_PROCFS");
+				     "stp_probe_type_procfs");
 
       // We've got 2 problems here.  The data count could be greater
       // than MAXSTRINGLEN or greater than the bufsize (if the same
@@ -439,8 +439,8 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
           fname = "_procfs_value_get";
           ec->code = string("    struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string("); /* pure */\n")
 
-            + string("    _stp_copy_from_user(THIS->__retvalue, data->buffer, data->count);\n")
-            + string("    THIS->__retvalue[data->count] = '\\0';\n");
+            + string("    _stp_copy_from_user(STAP_RETVALUE, data->buffer, data->count);\n")
+            + string("    STAP_RETVALUE[data->count] = '\\0';\n");
         }
       else					// lvalue
         {
@@ -448,14 +448,14 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
             {
               fname = "_procfs_value_set";
               ec->code = string("struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string(");\n")
-                + string("    strlcpy(data->buffer, THIS->value, data->bufsize);\n")
+                + string("    strlcpy(data->buffer, STAP_ARG_value, data->bufsize);\n")
                 + string("    data->count = strlen(data->buffer);\n");
             }
           else if (*op == ".=")
             {
               fname = "_procfs_value_append";
               ec->code = string("struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string(");\n")
-                + string("    strlcat(data->buffer, THIS->value, data->bufsize);\n")
+                + string("    strlcat(data->buffer, STAP_ARG_value, data->bufsize);\n")
                 + string("    data->count = strlen(data->buffer);\n");
             }
           else
