@@ -128,8 +128,8 @@ timer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->line() << ");";
   s.op->newline(-1) << "{";
   s.op->indent(1);
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "stp->probe",
-				 "_STP_PROBE_HANDLER_TIMER");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+				 "stp_probe_type_timer");
   s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op, true, s.suppress_handler_errors);
   s.op->newline(-1) << "}";
@@ -315,8 +315,8 @@ hrtimer_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline(-1) << "}";
   s.op->newline() << "{";
   s.op->indent(1);
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "stp->probe",
-				 "_STP_PROBE_HANDLER_HRTIMER");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp->probe",
+				 "stp_probe_type_hrtimer");
   s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op, true, s.suppress_handler_errors);
   s.op->newline(-1) << "}";
@@ -431,12 +431,12 @@ profile_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "static void enter_all_profile_probes (struct pt_regs *regs) {";
   s.op->newline(1) << "struct stap_probe * probe = "
                    << common_probe_init (probes[0]) << ";";
-  common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "probe",
-				 "_STP_PROBE_HANDLER_PROFILE_TIMER");
+  common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "probe",
+				 "stp_probe_type_profile_timer");
   // Timer interrupts save all registers, so if the interrupt happened
   // in user space we can rely on it being the full user pt_regs.
   s.op->newline() << "if (user_mode(regs)) {";
-  s.op->newline(1) << "c->probe_flags |= _STP_PROBE_STATE_USER_MODE;";
+  s.op->newline(1) << "c->user_mode_p = 1;";
   s.op->newline() << "c->uregs = regs;";
   s.op->newline(-1) << "} else {";
   s.op->newline(1) << "c->kregs = regs;";
@@ -501,13 +501,11 @@ profile_derived_probe_group::emit_module_exit (systemtap_session& s)
 {
   if (probes.empty()) return;
 
-  s.op->newline() << "for (i=0; i<" << probes.size() << "; i++)";
-  s.op->newline(1) << "#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)"; // == using_rpn of yore
+  s.op->newline() << "#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)"; // == using_rpn of yore
   s.op->newline() << "unregister_profile_notifier (& stap_profile_notifier);";
   s.op->newline() << "#else";
   s.op->newline() << "unregister_timer_hook (& enter_profile_probes);";
   s.op->newline() << "#endif";
-  s.op->indent(-1);
 }
 
 
