@@ -832,8 +832,8 @@ utrace_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->newline() << "static void stap_utrace_probe_handler(struct task_struct *tsk, struct stap_utrace_probe *p) {";
       s.op->indent(1);
 
-      common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "p->probe",
-				     "_STP_PROBE_HANDLER_UTRACE");
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "p->probe",
+				     "stp_probe_type_utrace");
 
       // call probe function
       s.op->newline() << "(*p->probe->ph) (c);";
@@ -859,10 +859,10 @@ utrace_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->indent(1);
       s.op->newline() << "struct stap_utrace_probe *p = (struct stap_utrace_probe *)engine->data;";
 
-      common_probe_entryfn_prologue (s.op, "STAP_SESSION_RUNNING", "p->probe",
-				     "_STP_PROBE_HANDLER_UTRACE_SYSCALL");
+      common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "p->probe",
+				     "stp_probe_type_utrace_syscall");
       s.op->newline() << "c->uregs = regs;";
-      s.op->newline() << "c->probe_flags |= _STP_PROBE_STATE_USER_MODE;";
+      s.op->newline() << "c->user_mode_p = 1;";
 
       // call probe function
       s.op->newline() << "(*p->probe->ph) (c);";
