@@ -401,8 +401,14 @@ tokenize_cxx(const string& str, vector<string>& tokens)
 // same policy as execvp().  A program name not containing a slash
 // will be searched along the $PATH.
 
+string find_executable(const string& name)
+{
+  const map<string, string> sysenv;
+  return find_executable(name, "", sysenv);
+}
+
 string find_executable(const string& name, const string& sysroot,
-		       map<string, string>& sysenv,
+		       const map<string, string>& sysenv,
 		       const string& env_path)
 {
   string retpath;
@@ -418,9 +424,9 @@ string find_executable(const string& name, const string& sysroot,
     }
   else // Nope, search $PATH.
     {
-      char *path;
+      const char *path;
       if (sysenv.count(env_path) != 0)
-        path = (char *)sysenv[env_path].c_str();
+        path = sysenv.find(env_path)->second.c_str();
       else
         path = getenv(env_path.c_str());
       if (path)
@@ -815,9 +821,14 @@ stap_system(int verbose, const string& description,
       ret = pid;
       if (pid > 0){
         ret = stap_waitpid(verbose, pid);
-        if(ret)
-          // XXX PR13274 needs-session to use print_warning()
-          clog << _F("WARNING: %s exited with status: %d", description.c_str(), ret) << endl;
+
+        // XXX PR13274 needs-session to use print_warning()
+        if (ret > 128)
+          clog << _F("WARNING: %s exited with signal: %d (%s)",
+                     description.c_str(), ret - 128, strsignal(ret - 128)) << endl;
+        else if (ret > 0)
+          clog << _F("WARNING: %s exited with status: %d",
+                     description.c_str(), ret) << endl;
       }
     }
 
@@ -870,11 +881,9 @@ void assert_regexp_match (const string& name, const string& value, const string&
   // run regexec
   int rc = regexec (r, value.c_str(), 0, 0, 0);
   if (rc)
-    {
-      cerr << _F("ERROR: Safety pattern mismatch for %s ('%s' vs. '%s') rc=%d",
-                 name.c_str(), value.c_str(), re.c_str(), rc) << endl;
-      exit(1);
-    }
+    throw runtime_error
+      (_F("ERROR: Safety pattern mismatch for %s ('%s' vs. '%s') rc=%d",
+          name.c_str(), value.c_str(), re.c_str(), rc));
 }
 
 
@@ -1028,11 +1037,30 @@ std::string autosprintf(const char* format, ...)
   va_start (args, format);
   int rc = vasprintf (&str, format, args);
   if (rc < 0)
-    return _F("autosprintf/vasprintf error %s", lex_cast(rc).c_str());
+    {
+      va_end(args);
+      return _F("autosprintf/vasprintf error %d", rc);
+    }
   string s = str;
   va_end (args);
   free (str);
   return s; /* by copy */
 }
 
+std::string
+get_self_path()
+{
+  char buf[1024]; // This really should be enough for anybody...
+  const char *file = "/proc/self/exe";
+  ssize_t len = readlink(file, buf, sizeof(buf) - 1);
+  if (len > 0)
+    {
+      buf[len] = '\0';
+      file = buf;
+    }
+  // otherwise the path is ridiculously large, fall back to /proc/self/exe.
+  //
+  return std::string(file);
+}
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
